# Copyright (C) 2023 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Tuple, Type

from pontos.version.errors import ProjectError

from .cmake import CMakeVersionCommand
from .go import GoVersionCommand
from .javascript import JavaScriptVersionCommand
from .python import PythonVersionCommand
from .version import VersionCommand

_COMMANDS: Tuple[Type[VersionCommand]] = (
    CMakeVersionCommand,
    GoVersionCommand,
    JavaScriptVersionCommand,
    PythonVersionCommand,
)


def gather_project() -> VersionCommand:
    """
    Get the fitting VersionCommand for a project in the current working
    directory

    Raises:
        ProjectError if no fitting VersionCommand could be found
    """
    for cmd in _COMMANDS:
        command = cmd()
        if command.project_found():
            return command

    raise ProjectError("No project settings file found")
